%% Inequlities to vertices conversion
%
%Ineq and Eq are structured parameters, see inequality.m.
%Ineq contains all inequalities in the form Ax <= b, Eq contains all equalities.
%Input parameter output is optional and if it is 'S',
%output is in the double representation (see manual)
%
%Output parameter Vertices is a list of vertices of the polyhedron
%specified by Ineq and Eq or polyhedron in double representation.
%Second output parametr is optional and contains list of all endless edges
%if there are any.
%
%[Vertices, EndlessEdges] = ineqtovertices(Ineq, Eq, output)

function [Vertices, EndlessEdges, memory] = ineqtovertices(Ineq, Eq, output)

%input tests
Vertices = [];
if nargin<2 || ~iscorrect(Ineq) || ~iscorrect(Eq)
	disp('Incorrect input, see manual');
	return
end %if
if isstruct(Ineq) && isstruct(Eq)
  if ~(columns(Ineq.A) == columns(Eq.A))
    disp('Input equalities and inequalities must defines same dimension.');
    return
  end %if
end %if
if ~(isstruct(Ineq))&& Ineq == [] && ~(isstruct(Eq)) && Eq == []
  disp('Empty input.')
  return;
end %if
if isstruct(Ineq)
  dim = columns(Ineq.A);
  Ineq = rmineq(Ineq);
end %if
if isstruct(Eq)
  dim = columns(Eq.A);
  Eq = rmeq(Eq);      
end %if

originalA=[];
if isstruct(Ineq)
  originalA = [Ineq.A,Ineq.b];
end %if
if isstruct(Eq)
  originalA = [originalA;Eq.A,Eq.b];
end %if

%inicialization
unbounded = 0;
EndlessEdges = [];
memory = 0;
queue=iniineqtovertices(Ineq,Eq,dim);
if ~queueisempty(queue)
	queuesize=1;
	v=[0,0];
	queue.v=vertexfromsimplextable(queue, dim);
	Vertices=queue.v;
  range = length(queue.nbs) + length(queue.bs);
  BVertices = emptytrie(range);
	BVertices=triepush(sort(queue.bs),BVertices,range);
  
  %main cycle
	while ~queueisempty(queue)
		[vertex,queue]=queuepop(queue,queuesize);
		for i=1:columns(vertex.mat)-1
			v(2)=i;
			v(1)=choosesmallest(vertex.mat(:,i),vertex.mat(:,end));
			if v(1)>0
      %column is positive, pivot exists
      %find new basis
				vertex2.mat=simplexonestep(vertex.mat,v);                                   
				vertex2.bs=vertex.bs;
				vertex2.nbs=vertex.nbs;
				helpik=vertex2.nbs(v(2));
				vertex2.nbs(v(2))=vertex2.bs(v(1));
				vertex2.bs(v(1))=helpik;
				sortbaze = sort(vertex2.bs);
				if ~intrie(sortbaze,BVertices)
        %new basis add to queue
					vertex2.v=vertexfromsimplextable(vertex2, dim);
					queue=queuepush(queue,vertex2);
					BVertices=triepush(sortbaze,BVertices,range);
					if ~containrow(Vertices,vertex2.v)
          %new vertex add to result
						Vertices=[Vertices;vertex2.v];
          end %if
        end %if
        
      %column is not positive, endless edge may exist (tested)
			elseif vertex.nbs(v(2))<=dim
        if ~isinlist(vertex.nbs(v(2)) + dim, vertex.bs)
          [unbounded,EndlessEdges(end+1,:)] = createendlessedge(v(2),vertex,originalA,dim);
        end %if
			elseif vertex.nbs(v(2))>=dim && vertex.nbs(v(2)) <= 2*dim
        if ~isinlist(vertex.nbs(v(2)) - dim, vertex.bs)
          [unbounded,EndlessEdges(end+1,:)] = createendlessedge(v(2),vertex,originalA,dim);
        end %if
      elseif vertex.nbs(v(2)) > 2*dim
        [unbounded,EndlessEdges(end+1,:)] = createendlessedge(v(2),vertex,originalA,dim);
      end %if
		end %for
    
    %memory measurement
    if memory < actmemory(whos)
      memory = actmemory(whos);
    end %if
	end %while
end %if

%final output formatting 
Vertices=realvertices(Vertices, originalA, dim);
if nargin > 2
	if output == 'S'
		Vertices = doublerepr(Vertices, originalA);
  end %if
end %if
if unbounded
	disp('Polyhedron is unbounded.');
  EndlessEdges = rmredudendlessedges(EndlessEdges);
end %if