%% Intersection
%
%Polyhedron1 and Polyhedron2 specifies input polyhedra for intersection.
%Each of them can be in the vertex, inequalities or double representation.
%For the format see manual.
%Input parameter output specifies in which form the output should be return.
%Possibilities: 'V' vertex repr., 'I' ineq. repr., 'S' structure of double repr.
%Result is polyherdon in the specified representation.
%EndlessEdges is optional and contains a list of all endless edges if there are any.
%
%[Result,Endlessedges] = intersection(Polyhedron1, Polyhedron2, output)

function [Result,Endlessedges] = intersection(Polyhedron1, Polyhedron2, output)

%input tests
Result = [];
if nargin<3
  disp('There are not enough input parameters.');
  return
end %if

%test Polyhedron1
d1 = kind(Polyhedron1);
if d1 == 'V'
	Polyhedron1 = verticestoineqcomb(Polyhedron1);
elseif d1 == 'S'
	Polyhedron1 = extractineq(Polyhedron1);
elseif d1 == 'F'
else
	disp('The first parametr is in incorrect form.');
	return;
end %if

%test Polyhedron2
d2 = kind(Polyhedron2);
if d2 == 'V'
	Polyhedron2 = verticestoineqcomb(Polyhedron2);
elseif d2 == 'S'
	Polyhedron2 = extractineq(Polyhedron2);
elseif d2 == 'F'
else
	disp('The second parametr is in incorrect form.');
	return;
end %if

%intersection
Prunik.A = [Polyhedron1.A;Polyhedron2.A];
Prunik.b = [Polyhedron1.b;Polyhedron2.b];
if output == 'V'
	[Result,Endlessedges] = ineqtovertices(Prunik,[]);
else
	[Result, Endlessedges] = ineqtovertices(Prunik,[],'S');
	if output == 'F'
		Result = extractineq(Result);
	elseif output == 'S'
	else 
		disp('Third parametr is incorrect, result is in double description.');
  end %if
end %if