%% Union
%
%Polyhedron1 and Polyhedron2 specifies input polyhedra for intersection.
%Each of them can be in the vertex, inequalities or double representation.
%For the format see manual.
%Input parameter output specifies in which form the output should be return.
%Possibilities: 'V' vertex repr., 'I' ineq. repr., 'S' structure of double repr.
%Result is polyherdon in the specified representation.
%EndlessEdges is optional and contains a list of all endless edges if there are any.
%
%[Result,Endlessedges] = union(Polyhedron1, Polyhedron2, output)

function [Result,Endlessedges] = union(Polyhedron1,Polyhedron2,output)

%inicialization
Result = [];
Endlessedges = [];

%test Polyhedron1
d1 = kind(Polyhedron1);
if d1 == 'F'
	[Polyhedron1, Endlessedges1] = ineqtovertices(Polyhedron1,[]);
  Endlessedges = Endlessedges1;
elseif d1 == 'S'
	Poly = [];
	for i = 1:length(Polyhedron1)
		Poly(i,:) = Polyhedron1(i).c;
	end %for
	Polyhedron1 = Poly;
elseif d1 == 'V'
else
	disp('The first parametr is in incorrect form.');
	return;
end %if

%test Polyhedron2
d2 = kind(Polyhedron2);
if d2 == 'F'
	[Polyhedron2, Endlessedges2] = ineqtovertices(Polyhedron2,[]);
  Endlessedges = Endlessedges2;
elseif d2 == 'S'
	Poly = [];
	for i = 1:length(Polyhedron2)
		Poly(i,:) = Polyhedron2(i).c;
	end %for
	Polyhedron2 = Poly;
elseif d2 == 'V'
else
	disp('The first parametr is in incorrect form.');
	return;
end %if

%remove redudant endless edges
if d1 == 'F' && d2 == 'F'
  Endlessedges = rmredudendlessedges([Endlessedges1;Endlessedges2]);
end %if

%input analyse
if columns(Polyhedron1) == columns(Polyhedron2)
  if columns(Polyhedron1) == 0
    disp('Both polyhedra are empty or have no vertex.');
    return
  else
    Union = [Polyhedron1; Polyhedron2];
    dim = columns(Polyhedron1);
  end %if
elseif columns(Polyhedron1) == 0 || columns(Polyhedron2) == 0
    Union = [Polyhedron1; Polyhedron2];
    dim = max(columns(Polyhedron1),columns(Polyhedron2));
else
  disp('Polyhedra are in different dimensions.');
end %if

%remove redudant vertices
resUnion = Union(1,:);
for i = 2:rows(Union)
  resUnion(end+1,:) = Union(i,:);
  for j = 1:rows(resUnion)-1
    if resUnion(j,:) == Union(i,:)
      resUnion(end,:) = [];
      break;
    end %if
  end %for 
end %for
Union = resUnion;

%union
Polyhedron = verticestoineqcomb(Union, 'S');
Inequalities = extractineq(Polyhedron);
Inequalities = [Inequalities.A,Inequalities.b];
Inequalities = rmineqaccendless(Endlessedges, Inequalities);

%vertex extraction
Vertices = [];
for i = 1:length(Polyhedron)
	Vertices(end+1,:) = Polyhedron(i).c;
end %for

Inequalities = endlessfacets(Inequalities, Endlessedges, Vertices,dim);
if output == 'F'
  Result = inequality(Inequalities(:,1:end-1),Inequalities(:,end));
elseif output == 'V'
  Result = realvertices(Vertices, Inequalities, dim);
elseif output == 'S'
  Result = doublerepr(realvertices(Vertices, Inequalities, dim), Inequalities);
else
  disp('Third parameter is incorrect.');
  return
end %if