%% Vertices to inequalities conversion (combinatorical)
%
%Vertices are matrix, where rows represents coordiates of vertices
%of a polytop.
%Input parameter output is optional and if it is 'S',
%output is in the double representation (see manual)
%
%Output parameter Inequalities is a list of inequalities
%represents facets of the polytop or polyhedron in double representation.
%
%Inequalities = verticestoineqcomb(Vertices,output)

function [Inequalities, memory, time] = verticestoineqcomb(Vertices,output)

%time measurement starting
tic

%inicialization
if ~affindependence(Vertices)
  disp('Vertices does not defines a polyeder of full dimension.');
  return
end %if
[Inequalities, Polyhedron, Vertices, memory] = inivertoineqcomb(Vertices);

% main cycle
for i = 1:rows(Vertices)
	[Inequalities, Polyhedron, m] = addvertex(Vertices(i,:), Inequalities, Polyhedron);
  %memory measurement
  if memory < m
    memory = m;
  end %if
end %for

%final output formatting
Help = [];
Zero = zeros(1,columns(Inequalities));
for q = 1:rows(Inequalities)
	if ~same(Inequalities(q,:), Zero)
		Help(end+1,:) = Inequalities(q,:);
    end %if
end %for
Inequalities = Help;

if nargin > 1
	Vertices = [];
	for j = 1:length(Polyhedron)
		if ~(isempty(Polyhedron(j).ineq))
			Vertices(end+1, :) = Polyhedron(j).c;
        end %if
	end %for
	Inequalities = doublerepr(Vertices, Inequalities);
else
  Inequalities = inequality(Inequalities(:,1:end-1), Inequalities(:,end));
end %if

time = toc;