%%Vertices to inequalities conversion (duality)
%
%Vertices are matrix, where rows represents coordiates of vertices
%of a polytop.
%Input parameter output is optional and if it is 'S',
%output is in the double representation (see manual)
%
%Output parameter Inequalities is a list of inequalities
%represents facets of the polytop or polyhedron in double representation.
%
%Inequalities = verticestoineqcomb(Vertices,output)

function [Inequalities, memory, time] = verticestoineqdual(Vertices,output)

%time measurement starting
tic

%inicialize
Inequalities = [];
Dual = [];
dim = columns(Vertices);
if ~affindependence(Vertices)
  disp('Vertices does not defines a polyeder of full dimension.');
  return
end %if
[Vertices,centroid,OriginalV] = inivertoineqdual(Vertices);

%dim = 1 -> trivial
if dim < 2
  Inequalities = Vertices;
  Polyhedron = centroid;
  return
end %if

%creating dual polyhedron
for i = 1:rows(Vertices)
  Dual(end+1,:) = createdual(Vertices(i,:));
end %for

%calling dual algorithm
Dual = inequality(Dual(:,1:end-1),Dual(:,end));
[Verticesdual, E, memory] = ineqtovertices(Dual,[]);

%from dual back to primar
%move origin to original coordinates
for i = 1:rows(Verticesdual)
  Inequalities(end+1,:) = createdual(Verticesdual(i,:));
  Inequalities(end,:) = movehyperplane(Inequalities(end,:),-centroid); 
end %for

%final output formatting
if nargin > 1
  Inequalities = doublerepr(OriginalV,Inequalities);
else
  Inequalities = inequality(Inequalities(:,1:end-1),Inequalities(:,end));
end %if 

%time measurement finishing
time = toc;